"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _gateway = require("@apollo/gateway");

var _apolloServerLambda = require("apollo-server-lambda");

var _lodash = _interopRequireDefault(require("lodash.get"));

var _buildHeaders = _interopRequireDefault(require("./buildHeaders"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function toBool(value) {
  if (typeof value === "string") {
    return value === "true";
  }

  return Boolean(value);
}

function getError(error) {
  let err = (0, _lodash.default)(error, "extensions.response.body.error");

  if (err) {
    return {
      error: err.message,
      stack: err.stack
    };
  }

  err = (0, _lodash.default)(error, "extensions.exception");

  if (err) {
    return {
      error: error.message,
      stack: err.stacktrace
    };
  }

  return {
    error: error.message
  };
}

class WebinyDataSource extends _gateway.RemoteGraphQLDataSource {
  constructor(_ref) {
    let {
      onServiceError
    } = _ref,
        config = (0, _objectWithoutProperties2.default)(_ref, ["onServiceError"]);
    super(config);
    this.onServiceError = onServiceError;
  }

  async process({
    request,
    context
  }) {
    try {
      return await super.process({
        request,
        context
      }).then(res => {
        if (res.errors) {
          res.errors.map(error => this.onServiceError(error));
        }

        return res;
      });
    } catch (error) {
      this.onServiceError(error);
      return Promise.reject(error);
    }
  }

}

class ApolloGatewayError extends Error {
  constructor(errors) {
    super("");
    (0, _defineProperty2.default)(this, "errors", void 0);
    this.errors = errors;
  }

}

var _default = params => ({
  name: "create-apollo-gateway",
  type: "create-apollo-gateway",

  async createGateway({
    plugins
  }) {
    const {
      server = {},
      services,
      handler = {}
    } = params;
    const dataSourceErrors = [];
    const gateway = new _gateway.ApolloGateway({
      debug: toBool(process.env.DEBUG),
      serviceList: services,

      buildService({
        name,
        url
      }) {
        return new WebinyDataSource({
          url,

          willSendRequest(params) {
            const {
              context,
              request
            } = params; // TODO: process plugins

            if (context.headers) {
              Object.keys(context.headers).forEach(key => {
                if (context.headers[key]) {
                  request.http.headers.set(key, context.headers[key]);
                }
              });
            }
          },

          onServiceError(error) {
            dataSourceErrors.push(_objectSpread({}, getError(error), {
              service: name,
              url
            }));
          }

        });
      }

    });

    try {
      const {
        schema,
        executor
      } = await gateway.load();

      if (dataSourceErrors.length > 0) {
        throw new ApolloGatewayError(dataSourceErrors);
      }

      const apollo = new _apolloServerLambda.ApolloServer({
        schema,
        executor,
        introspection: toBool(server.introspection),
        playground: toBool(server.playground),
        context: async ({
          event
        }) => {
          const headers = (0, _buildHeaders.default)(event);
          const headerPlugins = plugins.byType("apollo-gateway-headers");
          headerPlugins.forEach(pl => pl.buildHeaders({
            headers,
            plugins
          }));
          return {
            headers
          };
        }
      });
      return apollo.createHandler({
        cors: _objectSpread({
          origin: "*",
          methods: "GET,HEAD,POST,OPTIONS"
        }, handler.cors)
      });
    } catch (e) {
      if (dataSourceErrors.length > 0) {
        console.log("dataSourceErrors", JSON.stringify({
          dataSourceErrors
        }, null, 2));
        throw new ApolloGatewayError(dataSourceErrors);
      }

      throw e;
    }
  }

});

exports.default = _default;
//# sourceMappingURL=index.js.map